@@ -238,7 +238,8 @@ DEFAULTS = {
         'dotty': opt_program('', ('dotty',)),
         'dot': opt_program('', ('dot',)),
         'ignore_missing_metadata': opt_boolean('no'),
-        'report_tool_options': opt_string('')
+        'report_tool_options': opt_string(''),
+        'obscure_pattern': opt_string('passw*')
     },
     'path': {
         'sharedir': opt_dir('%(datadir)s/crmsh'),
@@ -277,9 +278,10 @@ DEFAULTS = {
         'from_time': opt_string('-12H'),
         'compress': opt_boolean('yes'),
         'speed_up': opt_boolean('no'),
-        'collect_extra_logs': opt_string('/var/log/messages /var/log/pacemaker/pacemaker.log /var/log/pacemaker.log /var/log/ha-cluster-bootstrap.log'),
+        'collect_extra_logs': opt_string('/var/log/messages /var/log/pacemaker/pacemaker.log /var/log/pacemaker.log /var/log/crmsh/ha-cluster-bootstrap.log'),
         'remove_exist_dest': opt_boolean('no'),
-        'single_node': opt_boolean('no')
+        'single_node': opt_boolean('no'),
+        'sanitize_rule': opt_string('passw.*')
     }
 }
 
@@ -303,23 +305,34 @@ class _Configuration(object):
         self._systemwide = None
         self._user = None
 
+    def _safe_read(self, config_parser_inst, file_list):
+        """
+        Try to handle configparser.MissingSectionHeaderError while reading
+        """
+        from . import utils
+        try:
+            config_parser_inst.read(file_list)
+        except configparser.MissingSectionHeaderError:
+            with utils.disable_exception_traceback():
+                raise
+
     def load(self):
-        self._defaults = configparser.SafeConfigParser()
+        self._defaults = configparser.ConfigParser()
         for section, keys in DEFAULTS.items():
             self._defaults.add_section(section)
             for key, opt in keys.items():
                 self._defaults.set(section, key, opt.default)
 
         if os.path.isfile(_SYSTEMWIDE):
-            self._systemwide = configparser.SafeConfigParser()
-            self._systemwide.read([_SYSTEMWIDE])
+            self._systemwide = configparser.ConfigParser()
+            self._safe_read(self._systemwide, [_SYSTEMWIDE])
         # for backwards compatibility with <=2.1.1 due to ridiculous bug
         elif os.path.isfile("/etc/crm/crmsh.conf"):
-            self._systemwide = configparser.SafeConfigParser()
-            self._systemwide.read(["/etc/crm/crmsh.conf"])
+            self._systemwide = configparser.ConfigParser()
+            self._safe_read(self._systemwide, ["/etc/crm/crmsh.conf"])
         if os.path.isfile(_PERUSER):
-            self._user = configparser.SafeConfigParser()
-            self._user.read([_PERUSER])
+            self._user = configparser.ConfigParser()
+            self._safe_read(self._user, [_PERUSER])
 
     def save(self):
         if self._user:
@@ -351,7 +364,7 @@ class _Configuration(object):
             raise ValueError("Setting invalid option %s.%s" % (section, name))
         DEFAULTS[section][name].validate(value)
         if self._user is None:
-            self._user = configparser.SafeConfigParser()
+            self._user = configparser.ConfigParser()
         if not self._user.has_section(section):
             self._user.add_section(section)
         self._user.set(section, name, _stringify(value))
@@ -369,7 +382,7 @@ class _Configuration(object):
 
     def reset(self):
         '''reset to what is on disk'''
-        self._user = configparser.SafeConfigParser()
+        self._user = configparser.ConfigParser()
         self._user.read([_PERUSER])
 
 
