@@ -84,7 +84,7 @@ class Text(object):
 
     @staticmethod
     def isa(obj):
-        return isinstance(obj, basestring) or isinstance(obj, Text)
+        return isinstance(obj, str) or isinstance(obj, Text)
 
     def __init__(self, script, text, kind=None):
         self.script = script
@@ -98,7 +98,7 @@ class Text(object):
         val = self.text
         if val in (True, False):
             return "true" if val else "false"
-        if not isinstance(val, basestring):
+        if not isinstance(val, str):
             return str(val)
         return handles.parse(val, self.script.get('__values__', {})).strip()
 
@@ -118,6 +118,24 @@ class Text(object):
         return str(self) == str(obj)
 
 
+class WhenExpr(object):
+    def __init__(self, script, prog):
+        self.script = script
+        self.prog = prog
+
+    def __repr__(self):
+        return repr(self.prog)
+
+    def __str__(self):
+        lenv = self.script.get('__values__', {})
+        inp = handles.parse(self.prog, lenv).strip()
+        try:
+            from .minieval import minieval, InvalidExpression
+            return str(minieval(inp, lenv)).lower()
+        except InvalidExpression as err:
+            raise ValueError(str(err))
+
+
 def _strip(desc):
     if desc is None:
         return None
@@ -183,7 +201,7 @@ class Actions(object):
                 action['value'] = value
 
             def arrow(v):
-                return ' -> '.join(v.items()[0])
+                return ' -> '.join(list(v.items())[0])
             action['text'] = '\n'.join([arrow(x) for x in value])
         elif name == 'cib' or name == 'crm':
             action['text'] = str(Text.cib(script, value))
@@ -206,17 +224,18 @@ class Actions(object):
             action['longdesc'] = Text.desc(script, action['longdesc'])
 
         hre = handles.headmatcher
+        ident_re = re.compile(r'([a-z_-][a-z0-9_-]*)$', re.IGNORECASE)
 
         if 'when' in action:
             when = action['when']
-            if hre.search(when):
-                action['when'] = Text(script, when)
-            elif when:
+            if ident_re.match(when):
                 action['when'] = Text(script, '{{%s}}' % (when))
+            elif when:
+                action['when'] = WhenExpr(script, when)
             else:
                 del action['when']
-        for k, v in action.iteritems():
-            if isinstance(v, basestring) and hre.search(v):
+        for k, v in action.items():
+            if isinstance(v, str) and hre.search(v):
                 v = Text(script, v)
             if Text.isa(v):
                 action[k] = str(v).strip()
@@ -343,7 +362,7 @@ class Actions(object):
         input: list of packages
         or: map of <os>: <packages>
         """
-        self._run.execute_shell(self._nodes or 'all', '''#!/usr/bin/env python
+        self._run.execute_shell(self._nodes or 'all', '''#!/usr/bin/env python3
 import crm_script
 import crm_init
 
@@ -354,10 +373,10 @@ crm_script.exit_ok(True)
     def service(self):
         values = []
         for s in self._value:
-            for v in s.iteritems():
+            for v in s.items():
                 values.append(v)
         services = "\n".join([('crm_script.service%s' % repr(v)) for v in values])
-        self._run.execute_shell(self._nodes or 'all', '''#!/usr/bin/env python
+        self._run.execute_shell(self._nodes or 'all', '''#!/usr/bin/env python3
 import crm_script
 import crm_init
 
@@ -375,12 +394,13 @@ crm_script.exit_ok(True)
         include.
         """
 
+
 _actions = dict([(n, getattr(Actions, n)) for n in dir(Actions) if not n.startswith('_')])
 
 
 def _find_action(action):
     """return name of action for action"""
-    for a in _actions.keys():
+    for a in list(_actions.keys()):
         if a in action:
             return a
     return None
@@ -408,7 +428,7 @@ def _parse_yaml(scriptname, scriptfile):
     try:
         import yaml
         with open(scriptfile) as f:
-            data = yaml.load(f)
+            data = yaml.load(f, Loader=yaml.SafeLoader)
             if isinstance(data, list):
                 data = data[0]
     except ImportError as e:
@@ -601,7 +621,7 @@ def _parse_hawk_workflow(scriptname, scriptfile):
 
     if config.core.debug:
         import pprint
-        print "Parsed hawk workflow:"
+        print("Parsed hawk workflow:")
         pprint.pprint(data)
     return data
 
@@ -675,7 +695,7 @@ def _make_cib_for_agent(name, agent, data, ops):
 
 
 def _merge_objects(o1, o2):
-    for key, value in o2.iteritems():
+    for key, value in o2.items():
         o1[key] = value
 
 
@@ -745,7 +765,7 @@ def _process_agent_include(script, include):
         # Make any overriden parameters non-advanced
         # unless explicitly set to advanced
         pobj['advanced'] = False
-        for key, value in param.iteritems():
+        for key, value in param.items():
             if key in ('shortdesc', 'longdesc'):
                 pobj[key] = value
             elif key == 'value':
@@ -816,7 +836,7 @@ def _process_script_include(script, include):
     def _merge_step_param(step, param):
         for p in step.get('parameters', []):
             if p['name'] == param['name']:
-                for key, value in param.iteritems():
+                for key, value in param.items():
                     if key in ('shortdesc', 'longdesc'):
                         p[key] = value
                     elif key == 'value' and Text.isa(value):
@@ -865,7 +885,7 @@ def _process_include(script, include):
     elif 'script' in include:
         return _process_script_include(script, include)
     else:
-        raise ValueError("Unknown include type: %s" % (', '.join(include.keys())))
+        raise ValueError("Unknown include type: %s" % (', '.join(list(include.keys()))))
 
 
 def _postprocess_script_step(script, step):
@@ -876,7 +896,7 @@ def _postprocess_script_step(script, step):
     step['longdesc'] = step.get('longdesc', '')
     for p in step.get('parameters', []):
         if 'name' not in p:
-            raise ValueError("Parameter has no name: %s" % (p.keys()))
+            raise ValueError("Parameter has no name: %s" % (list(p.keys())))
         p['shortdesc'] = _strip(p.get('shortdesc', ''))
         p['longdesc'] = p.get('longdesc', '')
         if 'default' in p and 'value' not in p:
@@ -885,7 +905,7 @@ def _postprocess_script_step(script, step):
         if 'value' in p:
             if p['value'] is None:
                 del p['value']
-            elif isinstance(p['value'], basestring):
+            elif isinstance(p['value'], str):
                 p['value'] = Text(script, p['value'])
         if 'required' not in p:
             p['required'] = False
@@ -1006,16 +1026,16 @@ def load_script_file(script, filename):
     obj = _postprocess_script(parsed)
     if 'name' in obj:
         script = obj['name']
-    if script not in _script_cache or isinstance(_script_cache[script], basestring):
+    if script not in _script_cache or isinstance(_script_cache[script], str):
         _script_cache[script] = obj
     return obj
 
 
 def load_script_string(script, yml):
     build_script_cache()
-    import cStringIO
+    import io
     import yaml
-    data = yaml.load(cStringIO.StringIO(yml))
+    data = yaml.load(io.StringIO(yml), Loader=yaml.SafeLoader)
     if isinstance(data, list):
         data = data[0]
     if 'parameters' in data:
@@ -1036,10 +1056,10 @@ def load_script_string(script, yml):
 def load_script(script):
     build_script_cache()
     if script not in _script_cache:
-        common_debug("cache: %s" % (_script_cache.keys()))
+        common_debug("cache: %s" % (list(_script_cache.keys())))
         raise ValueError("Script not found: %s" % (script))
     s = _script_cache[script]
-    if isinstance(s, basestring):
+    if isinstance(s, str):
         try:
             return load_script_file(script, s)
         except KeyError as err:
@@ -1064,7 +1084,7 @@ def _check_control_persist():
     '''
     cmd = 'ssh -o ControlPersist'.split()
     if options.regression_tests:
-        print(".EXT", cmd)
+        print((".EXT", cmd))
     cmd = subprocess.Popen(cmd,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
@@ -1120,7 +1140,7 @@ def _print_debug(printer, local_node, hosts, workdir, opts):
     if hosts:
         for host, result in _parallax_call(printer, hosts,
                                            "if [ -f '%s' ]; then cat '%s'; fi" % (dbglog, dbglog),
-                                           opts).iteritems():
+                                           opts).items():
             if isinstance(result, parallax.Error):
                 printer.error(host, result)
             else:
@@ -1148,7 +1168,7 @@ def _run_cleanup(printer, has_remote_actions, local_node, hosts, workdir, opts):
         for host, result in _parallax_call(printer, hosts,
                                            "%s %s" % (cleanscript,
                                                       workdir),
-                                           opts).iteritems():
+                                           opts).items():
             if isinstance(result, parallax.Error):
                 printer.error(host, "Clean: %s" % (result))
             else:
@@ -1309,7 +1329,7 @@ def _verify_type(param, value, errors):
             errors.append("%s=%s enum without list of values" % (param.get('name'), value))
         else:
             opts = param['values']
-            if isinstance(opts, basestring):
+            if isinstance(opts, str):
                 opts = opts.replace(',', ' ').split(' ')
             for v in opts:
                 if value.lower() == v.lower():
@@ -1339,6 +1359,7 @@ def _verify_type(param, value, errors):
         errors.append("%s=%s is unknown type %s" % (param.get('name'), value, vtype))
     return value
 
+
 _NO_RESOLVE = object()
 
 
@@ -1378,7 +1399,7 @@ def _resolve_params(step, params, path, errors):
     """
     ret = {}
 
-    for pname, pvalue in params.iteritems():
+    for pname, pvalue in params.items():
         result = _resolve_direct(step, pname, pvalue, path, errors)
         if result is not _NO_RESOLVE:
             ret[pname] = result
@@ -1419,7 +1440,7 @@ def _check_parameters(script, params):
 
     def _split_commons(params):
         ret, cdict = {}, dict([(c, d) for c, d, _ in common_params()])
-        for key, value in params.iteritems():
+        for key, value in params.items():
             if key in cdict:
                 cdict[key] = value
             else:
@@ -1432,7 +1453,7 @@ def _check_parameters(script, params):
     if errors:
         raise ValueError('\n'.join(errors))
 
-    for key, value in commons.iteritems():
+    for key, value in commons.items():
         params[key] = value
 
     def _fill_values(path, into, source, srcreq):
@@ -1489,7 +1510,7 @@ def _handles_values(ret, script, params, subactions):
         context: source step
         params: values for step
         """
-        for key, value in params.iteritems():
+        for key, value in params.items():
             if not isinstance(value, dict):
                 to[key] = value
 
@@ -1576,7 +1597,7 @@ def _create_script_workdir(script, workdir):
             else:
                 cmd = ["mkdir", "-p", workdir]
             if options.regression_tests:
-                print ".EXT", cmd
+                print(".EXT", cmd)
             if subprocess.call(cmd, shell=False) != 0:
                 raise ValueError("Failed to create temporary working directory")
             # only copytree if script is a dir
@@ -1585,10 +1606,10 @@ def _create_script_workdir(script, workdir):
         else:
             cmd = ["mkdir", "-p", workdir]
             if options.regression_tests:
-                print ".EXT", cmd
+                print(".EXT", cmd)
             if subprocess.call(cmd, shell=False) != 0:
                 raise ValueError("Failed to create temporary working directory")
-    except (IOError, OSError), e:
+    except (IOError, OSError) as e:
         raise ValueError(e)
 
 
@@ -1600,7 +1621,7 @@ def _copy_utils(dst):
         import glob
         for f in glob.glob(os.path.join(config.path.sharedir, 'utils/*.py')):
             shutil.copy(f, dst)
-    except (IOError, OSError), e:
+    except (IOError, OSError) as e:
         raise ValueError(e)
 
 
@@ -1609,7 +1630,7 @@ def _create_remote_workdirs(printer, hosts, path, opts):
     ok = True
     for host, result in _parallax_call(printer, hosts,
                                        "mkdir -p %s" % (os.path.dirname(path)),
-                                       opts).iteritems():
+                                       opts).items():
         if isinstance(result, parallax.Error):
             printer.error(host, "Start: %s" % (result))
             ok = False
@@ -1624,7 +1645,7 @@ def _copy_to_remote_dirs(printer, hosts, path, opts):
     ok = True
     for host, result in _parallax_copy(printer, hosts,
                                        path,
-                                       path, opts).iteritems():
+                                       path, opts).items():
         if isinstance(result, parallax.Error):
             printer.debug("_copy_to_remote_dirs failed: %s, %s, %s" % (hosts, path, opts))
             printer.error(host, result)
@@ -1655,7 +1676,7 @@ def _copy_to_all(printer, workdir, hosts, local_node, src, dst, opts):
     """
     ok = True
     ret = _parallax_copy(printer, hosts, src, dst, opts)
-    for host, result in ret.iteritems():
+    for host, result in ret.items():
         if isinstance(result, parallax.Error):
             printer.error(host, result)
             ok = False
@@ -1708,7 +1729,7 @@ def clean_steps(steps):
 
 
 def clean_run_params(params):
-    for key, value in params.iteritems():
+    for key, value in params.items():
         if isinstance(value, dict):
             clean_run_params(value)
         elif Text.isa(value):
@@ -1845,7 +1866,7 @@ class RunActions(object):
                 self.result = ''
             self.data.append(self.result)
             if isinstance(self.result, dict):
-                for k, v in self.result.iteritems():
+                for k, v in self.result.items():
                     self.data[0][k] = v
             self.rc = True
         else:
@@ -1950,7 +1971,7 @@ class RunActions(object):
                 f.write(s)
                 if not s.endswith('\n'):
                     f.write("\n")
-        except IOError, msg:
+        except IOError as msg:
             self.printer.error(self.local_node_name(), "Write failed: %s" % (msg))
             return
         return fn
@@ -1976,12 +1997,13 @@ class RunActions(object):
         for host, result in _parallax_call(self.printer,
                                            self.hosts,
                                            cmdline,
-                                           self.opts).iteritems():
+                                           self.opts).items():
             if isinstance(result, parallax.Error):
                 self.printer.error(host, "Remote error: %s" % (result))
                 ok = False
             else:
                 rc, out, err = result
+                out = utils.to_ascii(out)
                 if rc != 0:
                     self.printer.error(host, "Remote error (rc=%s) %s%s" % (rc, out, err))
                     ok = False
@@ -2075,7 +2097,7 @@ def run(script, params, printer):
         else:
             return runner.all_actions()
 
-    except (OSError, IOError), e:
+    except (OSError, IOError) as e:
         import traceback
         traceback.print_exc()
         raise ValueError("Internal error while running %s: %s" % (name, e))
@@ -2123,7 +2145,7 @@ def _process_actions(script, params):
     for action in actions:
         name = _find_action(action)
         if name is None:
-            raise ValueError("Unknown action: %s" % (action.keys()))
+            raise ValueError("Unknown action: %s" % (list(action.keys())))
         action['name'] = name
         toadd = []
         if name == 'include':
@@ -2167,7 +2189,7 @@ def verify(script, params, external_check=True):
         cmd.extend(["verify", "commit", "\n"])
         try:
             common_debug("Try executing %s" % ("\n".join(cmd)))
-            rc, out = utils.filter_string(['crm', '-f', '-', 'configure'], "\n".join(cmd), stderr_on='stdout', shell=False)
+            rc, out = utils.filter_string(['crm', '-f', '-', 'configure'], "\n".join(cmd).encode('utf-8'), stderr_on='stdout', shell=False)
             errm = re.compile(r"^ERROR: \d+: (.*)$")
             outp = []
             for l in (out or "").splitlines():
@@ -2187,6 +2209,6 @@ def verify(script, params, external_check=True):
 
 
 def _make_boolean(v):
-    if isinstance(v, basestring):
+    if isinstance(v, str):
         return utils.get_boolean(v)
     return v not in (0, False, None)
