@@ -2,8 +2,9 @@
 # Copyright (C) 2013 Kristoffer Gronlund <kgronlund@suse.com>
 # See COPYING for license information.
 
-import optparse
+import sys
 import re
+from argparse import ArgumentParser, RawDescriptionHelpFormatter
 from . import command
 from . import utils
 from .msg import err_buf
@@ -14,11 +15,23 @@ from . import corosync
 from .cibconfig import cib_factory
 
 
-class OptParser(optparse.OptionParser):
+class ArgParser(ArgumentParser):
     def format_epilog(self, formatter):
         return self.epilog or ""
 
 
+def parse_options(parser, args):
+    try:
+        options, args = parser.parse_known_args(list(args))
+    except:
+        return None, None
+    if options.help:
+        parser.print_help()
+        return None, None
+    utils.check_space_option_value(options)
+    return options, args
+
+
 def _remove_completer(args):
     try:
         n = utils.list_cluster_nodes()
@@ -42,7 +55,7 @@ def script_args(args):
 
 def get_cluster_name():
     cluster_name = None
-    if not bootstrap.service_is_active("corosync.service"):
+    if not utils.service_is_active("corosync.service"):
         name = corosync.get_values('totem.cluster_name')
         if name:
             cluster_name = name[0]
@@ -79,6 +92,9 @@ class Cluster(command.UI):
         Starts the cluster services on this node
         '''
         try:
+            if utils.service_is_active("pacemaker.service"):
+                err_buf.info("Cluster services already started")
+                return
             utils.start_service("pacemaker")
             if utils.is_qdevice_configured():
                 utils.start_service("corosync-qdevice")
@@ -94,6 +110,9 @@ class Cluster(command.UI):
         Stops the cluster services on this node
         '''
         try:
+            if not utils.service_is_active("corosync.service"):
+                err_buf.info("Cluster services already stopped")
+                return
             if utils.is_qdevice_configured():
                 utils.stop_service("corosync-qdevice")
             utils.stop_service("corosync")
@@ -163,7 +182,7 @@ class Cluster(command.UI):
         if len(args) > 0:
             if '--dry-run' in args or looks_like_hostnames(args):
                 args = ['--yes', '--nodes'] + [arg for arg in args if arg != '--dry-run']
-        parser = OptParser(usage="usage: init [options] [STAGE]", epilog="""
+        parser = ArgParser(usage="init [options] [STAGE]", epilog="""
 
 Stage can be one of:
     ssh         Create SSH keys for passwordless SSH between cluster nodes
@@ -186,74 +205,66 @@ Note:
     To use storage you have already configured, pass -s and -o to specify
     the block devices for SBD and OCFS2, and the automatic partitioning
     will be skipped.
-""", add_help_option=False)
+""", add_help=False, formatter_class=RawDescriptionHelpFormatter)
 
-        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
-        parser.add_option("-q", "--quiet", action="store_true", dest="quiet",
-                          help="Be quiet (don't describe what's happening, just do it)")
-        parser.add_option("-y", "--yes", action="store_true", dest="yes_to_all",
-                          help='Answer "yes" to all prompts (use with caution, this is destructive, especially during the "storage" stage. The /root/.ssh/id_rsa key will be overwritten unless the option "--no-overwrite-sshkey" is used)')
-        parser.add_option("-t", "--template", dest="template",
-                          help='Optionally configure cluster with template "name" (currently only "ocfs2" is valid here)')
-        parser.add_option("-n", "--name", metavar="NAME", dest="name", default="hacluster",
-                          help='Set the name of the configured cluster.')
-        parser.add_option("-N", "--nodes", metavar="NODES", dest="nodes",
-                          help='Additional nodes to add to the created cluster. ' +
-                          'May include the current node, which will always be the initial cluster node.')
-        # parser.add_option("--quick-start", dest="quickstart", action="store_true", help="Perform basic system configuration (NTP, watchdog, /etc/hosts)")
-        parser.add_option("-S", "--enable-sbd", dest="diskless_sbd", action="store_true",
-                          help="Enable SBD even if no SBD device is configured (diskless mode)")
-        parser.add_option("-w", "--watchdog", dest="watchdog", metavar="WATCHDOG",
-                          help="Use the given watchdog device")
-        parser.add_option("--no-overwrite-sshkey", action="store_true", dest="no_overwrite_sshkey",
-                          help='Avoid "/root/.ssh/id_rsa" overwrite if "-y" option is used (False by default)')
+        parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_argument("-q", "--quiet", action="store_true", dest="quiet",
+                            help="Be quiet (don't describe what's happening, just do it)")
+        parser.add_argument("-y", "--yes", action="store_true", dest="yes_to_all",
+                            help='Answer "yes" to all prompts (use with caution, this is destructive, especially during the "storage" stage. The /root/.ssh/id_rsa key will be overwritten unless the option "--no-overwrite-sshkey" is used)')
+        parser.add_argument("-t", "--template", dest="template", metavar="TEMPLATE", choices=['ocfs2'],
+                            help='Optionally configure cluster with template "name" (currently only "ocfs2" is valid here)')
+        parser.add_argument("-n", "--name", metavar="NAME", dest="cluster_name", default="hacluster",
+                            help='Set the name of the configured cluster.')
+        parser.add_argument("-N", "--nodes", metavar="NODES", dest="nodes",
+                            help='Additional nodes to add to the created cluster. May include the current node, which will always be the initial cluster node.')
+        # parser.add_argument("--quick-start", dest="quickstart", action="store_true", help="Perform basic system configuration (NTP, watchdog, /etc/hosts)")
+        parser.add_argument("-S", "--enable-sbd", dest="diskless_sbd", action="store_true",
+                            help="Enable SBD even if no SBD device is configured (diskless mode)")
+        parser.add_argument("-w", "--watchdog", dest="watchdog", metavar="WATCHDOG",
+                            help="Use the given watchdog device")
+        parser.add_argument("--no-overwrite-sshkey", action="store_true", dest="no_overwrite_sshkey",
+                            help='Avoid "/root/.ssh/id_rsa" overwrite if "-y" option is used (False by default)')
 
-        network_group = optparse.OptionGroup(parser, "Network configuration", "Options for configuring the network and messaging layer.")
-        network_group.add_option("-i", "--interface", dest="nic", metavar="IF",
-                                 help="Bind to IP address on interface IF")
-        network_group.add_option("-u", "--unicast", action="store_true", dest="unicast",
-                                 help="Configure corosync to communicate over unicast (UDP), and not multicast. " +
-                                 "Default is multicast unless an environment where multicast cannot be used is detected.")
-        network_group.add_option("-A", "--admin-ip", dest="admin_ip", metavar="IP",
-                                 help="Configure IP address as an administration virtual IP")
-        network_group.add_option("-M", "--multi-heartbeats", action="store_true", dest="second_hb",
-                                 help="Configure corosync with second heartbeat line")
-        network_group.add_option("-I", "--ipv6", action="store_true", dest="ipv6",
-                                 help="Configure corosync use IPv6")
-        network_group.add_option("--qnetd-hostname",
-                                 dest="qdevice", metavar="HOST",
-                                 help="HOST or IP of the QNetd server to be used")
-        network_group.add_option("--qdevice-port",
-                                 dest="qdevice_port", metavar="PORT", type="int", default=5403,
-                                 help="TCP PORT of QNetd server(default:5403)")
-        network_group.add_option("--qdevice-algo",
-                                 dest="qdevice_algo", metavar="ALGORITHM", default="ffsplit",
-                                 help="QNetd decision ALGORITHM(ffsplit/lms, default:ffsplit)")
-        network_group.add_option("--qdevice-tie-breaker",
-                                 dest="qdevice_tie_breaker", metavar="TIE_BREAKER", default="lowest",
-                                 help="QNetd TIE_BREAKER(lowest/highest/valid_node_id, default:lowest)")
-        network_group.add_option("--qdevice-tls",
-                                 dest="qdevice_tls", metavar="TLS", default="on",
-                                 help="Whether using TLS on QDevice/QNetd(on/off/required, default:on)")
-        network_group.add_option("--qdevice-heuristics",
-                                 dest="qdevice_heuristics", metavar="COMMAND",
-                                 help="COMMAND to run with absolute path. For multiple commands, use \";\" to separate(details about heuristics can see man 8 corosync-qdevice)")
-        parser.add_option_group(network_group)
+        network_group = parser.add_argument_group("Network configuration", "Options for configuring the network and messaging layer.")
+        network_group.add_argument("-i", "--interface", dest="nic_list", metavar="IF", action="append", choices=utils.interface_choice(),
+                                   help="Bind to IP address on interface IF. Use -i second time for second interface")
+        network_group.add_argument("-u", "--unicast", action="store_true", dest="unicast",
+                                   help="Configure corosync to communicate over unicast (UDP), and not multicast. " +
+                                   "Default is multicast unless an environment where multicast cannot be used is detected.")
+        network_group.add_argument("-A", "--admin-ip", dest="admin_ip", metavar="IP",
+                                   help="Configure IP address as an administration virtual IP")
+        network_group.add_argument("-M", "--multi-heartbeats", action="store_true", dest="second_heartbeat",
+                                   help="Configure corosync with second heartbeat line")
+        network_group.add_argument("-I", "--ipv6", action="store_true", dest="ipv6",
+                                   help="Configure corosync use IPv6")
 
-        storage_group = optparse.OptionGroup(parser, "Storage configuration", "Options for configuring shared storage.")
-        storage_group.add_option("-p", "--partition-device", dest="shared_device", metavar="DEVICE",
-                                 help='Partition this shared storage device (only used in "storage" stage)')
-        storage_group.add_option("-s", "--sbd-device", dest="sbd_device", metavar="DEVICE", action="append",
-                                 help="Block device to use for SBD fencing, use \";\" as separator for multi path")
-        storage_group.add_option("-o", "--ocfs2-device", dest="ocfs2_device", metavar="DEVICE",
-                                 help='Block device to use for OCFS2 (only used in "vgfs" stage)')
-        parser.add_option_group(storage_group)
-        try:
-            options, args = parser.parse_args(list(args))
-        except:
-            return
-        if options.help:
-            parser.print_help()
+        qdevice_group = parser.add_argument_group("QDevice configuration", "Options for configuring QDevice and QNetd.")
+        qdevice_group.add_argument("--qnetd-hostname", dest="qnetd_addr", metavar="HOST",
+                                   help="HOST or IP of the QNetd server to be used")
+        qdevice_group.add_argument("--qdevice-port", dest="qdevice_port", metavar="PORT", type=int, default=5403,
+                                   help="TCP PORT of QNetd server(default:5403)")
+        qdevice_group.add_argument("--qdevice-algo", dest="qdevice_algo", metavar="ALGORITHM", default="ffsplit", choices=['ffsplit', 'lms'],
+                                   help="QNetd decision ALGORITHM(ffsplit/lms, default:ffsplit)")
+        qdevice_group.add_argument("--qdevice-tie-breaker", dest="qdevice_tie_breaker", metavar="TIE_BREAKER", default="lowest",
+                                   help="QNetd TIE_BREAKER(lowest/highest/valid_node_id, default:lowest)")
+        qdevice_group.add_argument("--qdevice-tls", dest="qdevice_tls", metavar="TLS", default="on", choices=['on', 'off', 'required'],
+                                   help="Whether using TLS on QDevice/QNetd(on/off/required, default:on)")
+        qdevice_group.add_argument("--qdevice-heuristics", dest="qdevice_heuristics", metavar="COMMAND",
+                                   help="COMMAND to run with absolute path. For multiple commands, use \";\" to separate(details about heuristics can see man 8 corosync-qdevice)")
+        qdevice_group.add_argument("--qdevice-heuristics-mode", dest="qdevice_heuristics_mode", metavar="MODE", choices=['on', 'sync', 'off'],
+                                   help="MODE of operation of heuristics(on/sync/off, default:sync)")
+
+        storage_group = parser.add_argument_group("Storage configuration", "Options for configuring shared storage.")
+        storage_group.add_argument("-p", "--partition-device", dest="shared_device", metavar="DEVICE",
+                                   help='Partition this shared storage device (only used in "storage" stage)')
+        storage_group.add_argument("-s", "--sbd-device", dest="sbd_devices", metavar="DEVICE", action="append",
+                                   help="Block device to use for SBD fencing, use \";\" as separator or -s multiple times for multi path (up to 3 devices)")
+        storage_group.add_argument("-o", "--ocfs2-device", dest="ocfs2_device", metavar="DEVICE",
+                                   help='Block device to use for OCFS2 (only used in "vgfs" stage)')
+
+        options, args = parse_options(parser, args)
+        if options is None or args is None:
             return
 
         stage = ""
@@ -261,46 +272,25 @@ Note:
             stage = args[0]
         if stage not in bootstrap.INIT_STAGES and stage != "":
             parser.error("Invalid stage (%s)" % (stage))
-            return False
 
-        if options.template and options.template != "ocfs2":
-            parser.error("Invalid template (%s)" % (options.template))
-            return False
+        if options.qnetd_addr:
+            if options.qdevice_heuristics_mode and not options.qdevice_heuristics:
+                parser.error("Option --qdevice-heuristics is required if want to configure heuristics mode")
+            options.qdevice_heuristics_mode = options.qdevice_heuristics_mode or "sync"
+        elif re.search("--qdevice-.*", ' '.join(sys.argv)) or stage == "qdevice":
+            parser.error("Option --qnetd-hostname is required if want to configure qdevice")
+
+        if options.sbd_devices and options.diskless_sbd:
+            parser.error("Can't use -s and -S options together")
 
         # if options.geo and options.name == "hacluster":
         #    parser.error("For a geo cluster, each cluster must have a unique name (use --name to set)")
-        #    return False
+        boot_context = bootstrap.Context.set_context(options)
+        boot_context.ui_context = context
+        boot_context.stage = stage
+        boot_context.args = args
 
-        qdevice = None
-        if options.qdevice:
-            qdevice = corosync.QDevice(
-                options.qdevice,
-                port=options.qdevice_port,
-                algo=options.qdevice_algo,
-                tie_breaker=options.qdevice_tie_breaker,
-                tls=options.qdevice_tls,
-                cmds=options.qdevice_heuristics)
-
-        bootstrap.bootstrap_init(
-            cluster_name=options.name,
-            ui_context=context,
-            nic=options.nic,
-            ocfs2_device=options.ocfs2_device,
-            shared_device=options.shared_device,
-            sbd_device=options.sbd_device,
-            diskless_sbd=options.diskless_sbd,
-            quiet=options.quiet,
-            template=options.template,
-            admin_ip=options.admin_ip,
-            yes_to_all=options.yes_to_all,
-            no_overwrite_sshkey=options.no_overwrite_sshkey,
-            unicast=options.unicast,
-            second_hb=options.second_hb,
-            ipv6=options.ipv6,
-            watchdog=options.watchdog,
-            qdevice=qdevice,
-            stage=stage,
-            args=args)
+        bootstrap.bootstrap_init(boot_context)
 
         # if options.geo:
         #    bootstrap.bootstrap_init_geo()
@@ -321,7 +311,7 @@ Note:
         '''
         Join this node to an existing cluster
         '''
-        parser = OptParser(usage="usage: join [options] [STAGE]", epilog="""
+        parser = ArgParser(usage="join [options] [STAGE]", epilog="""
 
 Stage can be one of:
     ssh         Obtain SSH keys from existing cluster node (requires -c <host>)
@@ -331,22 +321,18 @@ Stage can be one of:
     cluster     Start the cluster on this node
 
 If stage is not specified, each stage will be invoked in sequence.
-""", add_help_option=False)
-        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
-        parser.add_option("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
-        parser.add_option("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
-        parser.add_option("-w", "--watchdog", dest="watchdog", metavar="WATCHDOG", help="Use the given watchdog device")
+""", add_help=False, formatter_class=RawDescriptionHelpFormatter)
+        parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_argument("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
+        parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
+        parser.add_argument("-w", "--watchdog", dest="watchdog", metavar="WATCHDOG", help="Use the given watchdog device")
 
-        network_group = optparse.OptionGroup(parser, "Network configuration", "Options for configuring the network and messaging layer.")
-        network_group.add_option("-c", "--cluster-node", dest="cluster_node", help="IP address or hostname of existing cluster node", metavar="HOST")
-        network_group.add_option("-i", "--interface", dest="nic", help="Bind to IP address on interface IF", metavar="IF")
-        parser.add_option_group(network_group)
-        try:
-            options, args = parser.parse_args(list(args))
-        except:
-            return
-        if options.help:
-            parser.print_help()
+        network_group = parser.add_argument_group("Network configuration", "Options for configuring the network and messaging layer.")
+        network_group.add_argument("-c", "--cluster-node", dest="cluster_node", help="IP address or hostname of existing cluster node", metavar="HOST")
+        network_group.add_argument("-i", "--interface", dest="nic_list", metavar="IF", action="append", choices=utils.interface_choice(),
+                help="Bind to IP address on interface IF. Use -i second time for second interface")
+        options, args = parse_options(parser, args)
+        if options is None or args is None:
             return
 
         stage = ""
@@ -354,16 +340,12 @@ If stage is not specified, each stage will be invoked in sequence.
             stage = args[0]
         if stage not in ("ssh", "csync2", "ssh_merge", "cluster", ""):
             parser.error("Invalid stage (%s)" % (stage))
-            return False
 
-        bootstrap.bootstrap_join(
-            cluster_node=options.cluster_node,
-            ui_context=context,
-            nic=options.nic,
-            quiet=options.quiet,
-            yes_to_all=options.yes_to_all,
-            watchdog=options.watchdog,
-            stage=stage)
+        join_context = bootstrap.Context.set_context(options)
+        join_context.ui_context = context
+        join_context.stage = stage
+
+        bootstrap.bootstrap_join(join_context)
 
         return True
 
@@ -384,16 +366,13 @@ If stage is not specified, each stage will be invoked in sequence.
         Installs packages, sets up corosync and pacemaker, etc.
         Must be executed from a node in the existing cluster.
         '''
-        parser = OptParser(usage="usage: add [options] [node ...]", add_help_option=False)
-        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
-        parser.add_option("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
-        try:
-            options, args = parser.parse_args(list(args))
-        except:
-            return
-        if options.help:
-            parser.print_help()
+        parser = ArgParser(usage="add [options] [node ...]", add_help=False, formatter_class=RawDescriptionHelpFormatter)
+        parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
+        options, args = parse_options(parser, args)
+        if options is None or args is None:
             return
+
         for node in args:
             if not self._add_node(node, yes_to_all=options.yes_to_all):
                 return False
@@ -405,37 +384,29 @@ If stage is not specified, each stage will be invoked in sequence.
         '''
         Remove the given node(s) from the cluster.
         '''
-        parser = OptParser(usage="usage: remove [options] [<node> ...]", add_help_option=False)
-        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
-        parser.add_option("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
-        parser.add_option("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
-        parser.add_option("-c", "--cluster-node", dest="cluster_node", help="IP address or hostname of cluster node which will be deleted", metavar="HOST")
-        parser.add_option("-F", "--force", dest="force", help="Remove current node", action="store_true")
-        parser.add_option("--qdevice", dest="qdevice", help="Remove QDevice configuration and service from cluster", action="store_true")
-        try:
-            options, args = parser.parse_args(list(args))
-        except:
-            return
-        if options.help:
-            parser.print_help()
+        parser = ArgParser(usage="remove [options] [<node> ...]", add_help=False, formatter_class=RawDescriptionHelpFormatter)
+        parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_argument("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
+        parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
+        parser.add_argument("-c", "--cluster-node", dest="cluster_node", help="IP address or hostname of cluster node which will be deleted", metavar="HOST")
+        parser.add_argument("-F", "--force", dest="force", help="Remove current node", action="store_true")
+        parser.add_argument("--qdevice", dest="qdevice_rm_flag", help="Remove QDevice configuration and service from cluster", action="store_true")
+        options, args = parse_options(parser, args)
+        if options is None or args is None:
             return
+
         if options.cluster_node is not None and options.cluster_node not in args:
             args = list(args) + [options.cluster_node]
+
+        rm_context = bootstrap.Context.set_context(options)
+        rm_context.ui_context = context
+
         if len(args) == 0:
-            bootstrap.bootstrap_remove(
-                cluster_node=None,
-                ui_context=context,
-                quiet=options.quiet,
-                yes_to_all=options.yes_to_all,
-                qdevice=options.qdevice)
+            bootstrap.bootstrap_remove(rm_context)
         else:
             for node in args:
-                bootstrap.bootstrap_remove(
-                    cluster_node=node,
-                    ui_context=context,
-                    quiet=options.quiet,
-                    yes_to_all=options.yes_to_all,
-                    force=options.force)
+                rm_context.cluster_node = node
+                bootstrap.bootstrap_remove(rm_context)
         return True
 
     @command.skill_level('administrator')
@@ -443,7 +414,7 @@ If stage is not specified, each stage will be invoked in sequence.
         '''
         Rename the cluster.
         '''
-        if not bootstrap.service_is_active("corosync.service"):
+        if not utils.service_is_active("corosync.service"):
             context.fatal_error("Can't rename cluster when cluster service is stopped")
         old_name = cib_factory.get_property('cluster-name')
         if old_name and new_name == old_name:
@@ -492,7 +463,7 @@ If stage is not specified, each stage will be invoked in sequence.
         * arbitrator IP / hostname (optional)
         * list of tickets (can be empty)
         '''
-        parser = OptParser(usage="usage: geo-init [options]", epilog="""
+        parser = ArgParser(usage="geo-init [options]", epilog="""
 
 Cluster Description
 
@@ -507,19 +478,15 @@ Cluster Description
 
   Name clusters using the --name parameter to
   crm bootstrap init.
-""", add_help_option=False)
-        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
-        parser.add_option("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
-        parser.add_option("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
-        parser.add_option("-a", "--arbitrator", help="IP address of geo cluster arbitrator", dest="arbitrator", metavar="IP")
-        parser.add_option("-s", "--clusters", help="Geo cluster description (see details below)", dest="clusters", metavar="DESC")
-        parser.add_option("-t", "--tickets", help="Tickets to create (space-separated)", dest="tickets", metavar="LIST")
-        try:
-            options, args = parser.parse_args(list(args))
-        except:
-            return
-        if options.help:
-            parser.print_help()
+""", add_help=False, formatter_class=RawDescriptionHelpFormatter)
+        parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_argument("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
+        parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
+        parser.add_argument("-a", "--arbitrator", help="IP address of geo cluster arbitrator", dest="arbitrator", metavar="IP")
+        parser.add_argument("-s", "--clusters", help="Geo cluster description (see details below)", dest="clusters", metavar="DESC")
+        parser.add_argument("-t", "--tickets", help="Tickets to create (space-separated)", dest="tickets", metavar="LIST")
+        options, args = parse_options(parser, args)
+        if options is None or args is None:
             return
 
         if options.clusters is None:
@@ -537,7 +504,13 @@ Cluster Description
                 ticketlist = [t for t in re.split('[ ,;]+', options.tickets)]
             except ValueError:
                 parser.error("Invalid ticket list")
-        bootstrap.bootstrap_init_geo(options.quiet, options.yes_to_all, options.arbitrator, clustermap, ticketlist, ui_context=context)
+
+        geo_context = bootstrap.Context.set_context(options)
+        geo_context.clusters = clustermap
+        geo_context.tickets = ticketlist
+        geo_context.ui_context = context
+
+        bootstrap.bootstrap_init_geo(geo_context)
         return True
 
     @command.name("geo_join")
@@ -547,21 +520,17 @@ Cluster Description
         '''
         Join this cluster to a geo configuration.
         '''
-        parser = OptParser(usage="usage: geo-join [options]", add_help_option=False)
-        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
-        parser.add_option("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
-        parser.add_option("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
-        parser.add_option("-c", "--cluster-node", help="IP address of an already-configured geo cluster or arbitrator", dest="node", metavar="IP")
-        parser.add_option("-s", "--clusters", help="Geo cluster description (see geo-init for details)", dest="clusters", metavar="DESC")
-        try:
-            options, args = parser.parse_args(list(args))
-        except:
-            return
-        if options.help:
-            parser.print_help()
+        parser = ArgParser(usage="geo-join [options]", add_help=False, formatter_class=RawDescriptionHelpFormatter)
+        parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_argument("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
+        parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
+        parser.add_argument("-c", "--cluster-node", help="IP address of an already-configured geo cluster or arbitrator", dest="cluster_node", metavar="IP")
+        parser.add_argument("-s", "--clusters", help="Geo cluster description (see geo-init for details)", dest="clusters", metavar="DESC")
+        options, args = parse_options(parser, args)
+        if options is None or args is None:
             return
         errs = []
-        if options.node is None:
+        if options.cluster_node is None:
             errs.append("The --cluster-node argument is required.")
         if options.clusters is None:
             errs.append("The --clusters argument is required.")
@@ -570,7 +539,12 @@ Cluster Description
         clustermap = self._parse_clustermap(options.clusters)
         if clustermap is None:
             parser.error("Invalid cluster description format")
-        bootstrap.bootstrap_join_geo(options.quiet, options.yes_to_all, options.node, clustermap, ui_context=context)
+
+        geo_context = bootstrap.Context.set_context(options)
+        geo_context.clusters = clustermap
+        geo_context.ui_context = context
+
+        bootstrap.bootstrap_join_geo(geo_context)
         return True
 
     @command.name("geo_init_arbitrator")
@@ -580,19 +554,19 @@ Cluster Description
         '''
         Make this node a geo arbitrator.
         '''
-        parser = OptParser(usage="usage: geo-init-arbitrator [options]", add_help_option=False)
-        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
-        parser.add_option("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
-        parser.add_option("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
-        parser.add_option("-c", "--cluster-node", help="IP address of an already-configured geo cluster", dest="other", metavar="IP")
-        try:
-            options, args = parser.parse_args(list(args))
-        except:
+        parser = ArgParser(usage="geo-init-arbitrator [options]", add_help=False, formatter_class=RawDescriptionHelpFormatter)
+        parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_argument("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
+        parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
+        parser.add_argument("-c", "--cluster-node", help="IP address of an already-configured geo cluster", dest="cluster_node", metavar="IP")
+        options, args = parse_options(parser, args)
+        if options is None or args is None:
             return
-        if options.help:
-            parser.print_help()
-            return
-        bootstrap.bootstrap_arbitrator(options.quiet, options.yes_to_all, options.other, ui_context=context)
+
+        geo_context = bootstrap.Context.set_context(options)
+        geo_context.ui_context = context
+
+        bootstrap.bootstrap_arbitrator(geo_context)
         return True
 
     @command.completers_repeating(compl.call(scripts.param_completion_list, 'health'))
@@ -671,7 +645,11 @@ Cluster Description
 
         opts = parallax.Options()
         opts.ssh_options = ['StrictHostKeyChecking=no']
-        opts.askpass = utils.check_ssh_passwd_need(hosts)
+        for host in hosts:
+            res = utils.check_ssh_passwd_need(host)
+            if res:
+                opts.askpass = True
+                break
         for host, result in parallax.call(hosts, cmd, opts).items():
             if isinstance(result, parallax.Error):
                 err_buf.error("[%s]: %s" % (host, result))
