@@ -2,6 +2,7 @@
 # Copyright (C) 2013 Kristoffer Gronlund <kgronlund@suse.com>
 # See COPYING for license information.
 
+import re
 import time
 from . import command
 from . import completers as compl
@@ -550,6 +551,12 @@ class CibConfig(command.UI):
         "usage: show [xml] [<id>...]"
         from .utils import obscure
         osargs = [arg[8:] for arg in args if arg.startswith('obscure:')]
+        if not osargs and config.core.obscure_pattern:
+            # obscure_pattern could be
+            #   1. "pattern1 pattern2 pattern3"
+            #   2. "pattern1|pattern2|pattern3"
+            # regrex here also filter out possible spaces
+            osargs = re.split('\s*\|\s*|\s+', config.core.obscure_pattern.strip('|'))
         args = [arg for arg in args if not arg.startswith('obscure:')]
         with obscure(osargs):
             set_obj = mkset_obj(*args)
@@ -592,24 +599,50 @@ class CibConfig(command.UI):
     @command.skill_level('administrator')
     @command.completers(_id_list)
     def do_set(self, context, path, value):
-        "usage: set <path> <value>"
-        def split_path():
-            for oid in cib_factory.id_list():
-                if path.startswith(oid + "."):
-                    return oid, path[len(oid)+1:]
-            context.fatal_error("Invalid path: " + path)
-        obj_id, obj_attr = split_path()
+        """
+        usage: set <path> <value>
+
+        path:: id.[op_type.][interval.]name
+        """
+        path_errmsg = "Invalid path: \"{}\"; Valid path: \"id.[op_type.][interval.]name\"".format(path)
+        path_list = path.split('.')
+        if len(path_list) < 2 or len(path_list) > 4:
+            context.fatal_error(path_errmsg)
+
+        obj_id, *other_path_list = path_list
         rsc = cib_factory.find_object(obj_id)
         if not rsc:
-            context.fatal_error("Resource %s not found" % (obj_id))
-        nvpairs = rsc.node.xpath(".//nvpair[@name='%s']" % (obj_attr))
-        if not nvpairs:
-            context.fatal_error("Attribute not found: %s" % (path))
-        if len(nvpairs) != 1:
-            context.fatal_error("Expected 1 attribute named %s, found %s" %
-                                (obj_attr, len(nvpairs)))
+            context.fatal_error("Object {} not found".format(obj_id))
+
+        # Use case for: set id.name value
+        if len(other_path_list) == 1:
+            obj_attr = other_path_list[0]
+            nvpairs = rsc.node.xpath(".//nvpair[@name='{}']".format(obj_attr))
+            if not nvpairs:
+                context.fatal_error("Attribute not found: {}".format(path))
+            if len(nvpairs) != 1:
+                context.fatal_error("Expected 1 attribute named {}, found {}".format(obj_attr, len(nvpairs)))
+            nvpairs[0].set("value", value)
+
+        # Use case for: set id.op_type.name value
+        if len(other_path_list) == 2:
+            op_type, name = other_path_list
+            op_res = rsc.node.xpath(".//operations/op[@name='{}']".format(op_type))
+            if not op_res:
+                context.fatal_error("Operation \"{}\" not found for resource {}".format(op_type, obj_id))
+            if len(op_res) > 1:
+                context.fatal_error("Should specify interval of {}".format(op_type))
+            op_res[0].set(name, value)
+
+        # Use case for: set id.op_type.interval.name value
+        if len(other_path_list) == 3:
+            op_type, iv, name = other_path_list
+            op_res = rsc.node.xpath(".//operations/op[@id='{}-{}-{}']".format(obj_id, op_type, iv))
+            if not op_res:
+                context.fatal_error("Operation \"{}\" interval \"{}\" not found for resource {}".format(op_type, iv, obj_id))
+            op_res[0].set(name, value)
+
         rsc.set_updated()
-        nvpairs[0].set("value", value)
         return True
 
     @command.skill_level('administrator')
@@ -1038,7 +1071,7 @@ class CibConfig(command.UI):
                                   compl.call(schema.rng_attr_values, 'rsc_order', 'kind'),
                                   top_rsc_tmpl_id_list)
     def do_order(self, context, *args):
-        """usage: order <id> {kind|<score>}: <rsc>[:<action>] <rsc>[:<action>] ...
+        """usage: order <id> [kind]: <rsc>[:<action>] <rsc>[:<action>] ...
         [symmetrical=<bool>]"""
         return self.__conf_object(context.get_command_name(), *args)
 
