@@ -76,6 +76,9 @@ class Context(object):
                 self.command_info = self.current_level().get_child(token)
                 if not self.command_info:
                     self.fatal_error("No such command")
+                if self.command_name in self.command_info.aliases:
+                    common_warn("This command '{}' is deprecated, please use '{}'"\
+                            .format(self.command_name, self.command_info.name))
                 self.command_name = self.command_info.name
                 if self.command_info.type == 'level':
                     self.enter_level(self.command_info.level)
