@@ -15,18 +15,17 @@ import bz2
 import fnmatch
 import gc
 import ipaddress
+import argparse
 from contextlib import contextmanager, closing
 from . import config
 from . import userdir
 from . import constants
 from . import options
 from . import term
+from . import parallax
 from .msg import common_warn, common_info, common_debug, common_err, err_buf
 
 
-mcast_regrex = r'2(?:2[4-9]|3\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}'
-
-
 def to_ascii(input_str):
     """Convert the bytes string to a ASCII string
     Usefull to remove accent (diacritics)"""
@@ -92,94 +91,6 @@ def this_node():
     return os.uname()[1]
 
 
-def network_defaults(interface=None):
-    """
-    returns (interface, ip-address, network, prefix-length)
-    """
-    def valfor(l, key):
-        for i in range(0, len(l) - 1):
-            if l[i] == key:
-                return l[i + 1]
-        return None
-    _, outp = get_stdout("/sbin/ip -o route show")
-    info = [None, None, None, None]
-    if interface is not None:
-        info[0] = interface
-    for l in outp.splitlines():
-        sp = l.split()
-        if info[0] is None and len(sp) >= 5 and sp[0] == 'default' and sp[1] == 'via':
-            info[0] = sp[4]
-        if info[0] is not None and valfor(sp, 'dev') == info[0] and sp[0] != 'default':
-            src = valfor(sp, 'src')
-            if sp[0].find('/') >= 0:
-                nw, length = sp[0].split('/')
-                info[1], info[2], info[3] = src, nw, length
-                break
-            # we are reading /32 route entry
-            else:
-                info[1], info[2], info[3] = src, sp[0], 32
-    if info[0] is None:
-        raise ValueError("Failed to determine default network interface")
-    if info[1] is None:
-        ips = ip_in_local()
-        if len(ips) > 0:
-            info[1] = ips[0]
-    return tuple(info)
-
-
-def network_all(with_mask=False):
-    """
-    returns all networks on local node
-    """
-    all_networks = []
-    _, outp = get_stdout("/sbin/ip -o route show")
-    for l in outp.split('\n'):
-        if re.search(r'\.0/[0-9]+ ', l):
-            if with_mask:
-                all_networks.append(l.split()[0])
-            else:
-                all_networks.append(l.split('/')[0])
-    return all_networks
-
-
-def network_v6_all():
-    _, outp = get_stdout("/sbin/ip -6 -o addr show")
-    dict_ = {}
-    for line in outp.split('\n'):
-        if re.search(r' ::1/| [Ff][Ee]80:', line):
-            # skip local address and link-local address
-            continue
-        dict_[line.split()[1]] = []
-    for line in outp.split('\n'):
-        if re.search(r' ::1/| [Ff][Ee]80:', line):
-            # skip local address and link-local address
-            continue
-        dict_[line.split()[1]].append(line.split()[3])
-    return dict_
-
-
-def ip_in_local(IPv6=False):
-    # short-circuit ip list handling for
-    # cloud providers in order to use
-    # metadata service instead
-    ips = iplist_for_cloud()
-    if len(ips) > 0:
-        return ips
-
-    if not IPv6:
-        regex = r' [0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/[0-9]+ '
-    else:
-        regex = r' [0-9a-fA-F]{1,4}:.*:[0-9a-fA-F]{1,4}/[0-9]+ '
-
-    ip_local = []
-    _, outp = get_stdout("/sbin/ip addr show")
-    tmp = re.findall(regex, outp, re.M)
-    if tmp:
-        tmp = map(lambda x: x.split('/')[0].strip(), tmp)
-        ip_local += filter(lambda x: x != "127.0.0.1" and not re.search(r'^[Ff][Ee]80:', x), tmp)
-    return ip_local
-
-
 _cib_shadow = 'CIB_shadow'
 _cib_in_use = ''
 
@@ -815,7 +726,30 @@ def pipe_cmd_nosudo(cmd):
     return rc
 
 
-def get_stdout(cmd, input_s=None, stderr_on=True, shell=True):
+def run_cmd_on_remote(cmd, remote_addr, prompt_msg=None):
+    """
+    Run a cmd on remote node
+    return (rc, stdout, err_msg)
+    """
+    rc = 1
+    out_data = None
+    err_data = None
+
+    need_pw = check_ssh_passwd_need(remote_addr)
+    if need_pw and prompt_msg:
+        print(prompt_msg)
+    try:
+        result = parallax.parallax_call([remote_addr], cmd, need_pw)
+        rc, out_data, _ = result[0][1]
+    except ValueError as err:
+        err_match = re.search("Exited with error code ([0-9]+), Error output: (.*)", str(err))
+        if err_match:
+            rc, err_data = err_match.groups()
+    finally:
+        return int(rc), to_ascii(out_data), err_data
+
+
+def get_stdout(cmd, input_s=None, stderr_on=True, shell=True, raw=False):
     '''
     Run a cmd, return stdout output.
     Optional input string "input_s".
@@ -833,10 +767,12 @@ def get_stdout(cmd, input_s=None, stderr_on=True, shell=True):
                             stdout=subprocess.PIPE,
                             stderr=stderr)
     stdout_data, stderr_data = proc.communicate(input_s)
+    if raw:
+        return proc.returncode, stdout_data
     return proc.returncode, to_ascii(stdout_data).strip()
 
 
-def get_stdout_stderr(cmd, input_s=None, shell=True):
+def get_stdout_stderr(cmd, input_s=None, shell=True, raw=False):
     '''
     Run a cmd, return (rc, stdout, stderr)
     '''
@@ -848,6 +784,8 @@ def get_stdout_stderr(cmd, input_s=None, shell=True):
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
     stdout_data, stderr_data = proc.communicate(input_s)
+    if raw:
+        return proc.returncode, stdout_data, stderr_data
     return proc.returncode, to_ascii(stdout_data).strip(), to_ascii(stderr_data).strip()
 
 
@@ -1470,7 +1408,7 @@ def save_graphviz_file(ini_f, attr_d):
         common_err(msg)
         return False
     import configparser
-    p = configparser.SafeConfigParser()
+    p = configparser.ConfigParser()
     for section, sect_d in attr_d.items():
         p.add_section(section)
         for n, v in sect_d.items():
@@ -1492,7 +1430,7 @@ def load_graphviz_file(ini_f):
     if not os.path.isfile(ini_f):
         return True, None
     import configparser
-    p = configparser.SafeConfigParser()
+    p = configparser.ConfigParser()
     try:
         p.read(ini_f)
     except Exception as msg:
@@ -1750,6 +1688,15 @@ def list_corosync_nodes():
         return []
 
 
+def print_cluster_nodes():
+    """
+    Print the output of crm_node -l
+    """
+    rc, out, _ = get_stdout_stderr("crm_node -l")
+    if rc == 0 and out:
+        print("{}\n".format(out))
+
+
 def list_cluster_nodes():
     '''
     Returns a list of nodes in the cluster.
@@ -1791,6 +1738,29 @@ def list_cluster_nodes():
         raise ValueError("Error listing cluster nodes: %s" % (msg))
 
 
+def cluster_run_cmd(cmd):
+    """
+    Run cmd in cluster nodes
+    """
+    node_list = list_cluster_nodes()
+    if not node_list:
+        raise ValueError("Failed to get node list from cluster")
+    parallax.parallax_call(node_list, cmd)
+
+
+def list_cluster_nodes_except_me():
+    """
+    Get cluster node list and filter out self
+    """
+    node_list = list_cluster_nodes()
+    if not node_list:
+        raise ValueError("Failed to get node list from cluster")
+    me = this_node()
+    if me in node_list:
+        node_list.remove(me)
+    return node_list
+
+
 def service_info(name):
     p = is_program('systemctl')
     if p:
@@ -1809,45 +1779,6 @@ def service_info(name):
     return None
 
 
-def start_service(service, enable=False, start=True):
-    """
-    Start (and optionally enable) systemd service
-    TODO: support other init systems
-    """
-    if enable:
-        _rc, _out, _err = get_stdout_stderr("systemctl -q enable {}".format(service))
-    if start:
-        rc, _out, _err = get_stdout_stderr("systemctl -q is-active {}".format(service))
-        if rc != 0:
-            rc, _out, err = get_stdout_stderr("systemctl -q start {}".format(service))
-            if rc != 0:
-                raise IOError("systemd failed to start {}: {}".format(service, err))
-
-
-def stop_service(service, disable=False, stop=True):
-    """
-    Stop (and optionally disable) systemd service
-    TODO: support other init systems
-    """
-    rc, err = 0, ""
-    if stop:
-        rc, _out, err = get_stdout_stderr("systemctl -q stop {}".format(service))
-    if disable:
-        _rc, _out, _err = get_stdout_stderr("systemctl -q disable {}".format(service))
-    if rc != 0:
-        raise IOError("systemd failed to stop {}: {}".format(service, err))
-
-
-def enable_service(service, start=False):
-    "Enable (and optionally start) system service"
-    start_service(service, enable=True, start=start)
-
-
-def disable_service(service, stop=False):
-    "Enable (and optionally stop) system service"
-    stop_service(service, disable=True, stop=stop)
-
-
 def running_on(resource):
     "returns list of node names where the given resource is running"
     rsc_locate = "crm_resource --resource '%s' --locate"
@@ -2049,40 +1980,11 @@ def obscure(obscure_list):
         _obscured_nvpairs = prev
 
 
-def valid_ip_addr(addr, version=4):
-    import socket
-    try:
-        if version == 4:
-            socket.inet_pton(socket.AF_INET, addr)
-        elif version == 6:
-            socket.inet_pton(socket.AF_INET6, addr)
-        else:
-            return False
-    except socket.error:
-        return False
-    return True
-
-
-def get_ipv6_network(addr_with_mask):
-    return str(ipaddress.ip_interface(addr_with_mask).network.network_address)
-
-
 def gen_nodeid_from_ipv6(addr):
     return int(ipaddress.ip_address(addr)) % 1000000000
 
 
-def ip_in_network(addr, net):
-    return ipaddress.ip_address(addr) in ipaddress.ip_interface(net).network
-
-
-class IP:
-    def __init__(self, addr):
-        self.addr = ipaddress.ip_address(addr)
-
-    def version(self):
-        return self.addr.version
-
-# Set by detect_cloud() or iplist_for_cloud()
+# Set by detect_cloud()
 # to avoid multiple requests
 _ip_for_cloud = None
 
@@ -2129,7 +2031,7 @@ def detect_cloud():
     if not is_program("dmidecode"):
         return None
     rc, system_version = get_stdout("dmidecode -s system-version")
-    if re.search(r"\<.*\.amazon\>", system_version) is not None:
+    if re.search(r".*amazon.*", system_version) is not None:
         return "amazon-web-services"
     if rc != 0:
         return None
@@ -2154,24 +2056,6 @@ def detect_cloud():
     return None
 
 
-def iplist_for_cloud():
-    """
-    Return list of viable IPs for the
-    current cloud provider (if any)
-    """
-    global _ip_for_cloud
-    provider = detect_cloud()
-    if _ip_for_cloud is not None:
-        return [_ip_for_cloud]
-    if provider == "amazon-web-services":
-        uri = "http://169.254.169.254/latest/meta-data/local-ipv4"
-        result = _cloud_metadata_request(uri)
-        if result:
-            _ip_for_cloud = result
-            return [_ip_for_cloud]
-    return []
-
-
 def debug_timestamp():
     return datetime.datetime.now().strftime('%Y/%m/%d %H:%M:%S')
 
@@ -2190,21 +2074,32 @@ def get_member_iplist():
     return ip_list
 
 
-def check_ssh_passwd_need(hosts):
+def get_iplist_corosync_using():
+    """
+    Get ip list used by corosync
+    """
+    rc, out, err = get_stdout_stderr("corosync-cfgtool -s")
+    if rc != 0:
+        raise ValueError(err)
+    return re.findall(r'id\s*=\s*(.*)', out)
+
+
+def check_ssh_passwd_need(host):
+    """
+    Check whether access to host need password
+    """
     ssh_options = "-o StrictHostKeyChecking=no -o EscapeChar=none -o ConnectTimeout=15"
-    for host in hosts:
-        ssh_cmd = "ssh {} -T -o Batchmode=yes {} true".format(ssh_options, host)
-        rc, _, _ = get_stdout_stderr(ssh_cmd)
-        if rc != 0:
-            return True
-    return False
+    ssh_cmd = "ssh {} -T -o Batchmode=yes {} true".format(ssh_options, host)
+    rc, _, _ = get_stdout_stderr(ssh_cmd)
+    return rc != 0
 
 
-def check_port_open(host, port):
+def check_port_open(ip, port):
     import socket
 
-    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
-        if sock.connect_ex((host, port)) == 0:
+    family = socket.AF_INET6 if IP.is_ipv6(ip) else socket.AF_INET
+    with closing(socket.socket(family, socket.SOCK_STREAM)) as sock:
+        if sock.connect_ex((ip, port)) == 0:
             return True
         else:
             return False
@@ -2239,9 +2134,24 @@ def get_nodeinfo_from_cmaptool():
     return nodeid_ip_dict
 
 
+def get_iplist_from_name(name):
+    """
+    Given node host name, return this host's ip list in corosync cmap
+    """
+    ip_list = []
+    nodeid = get_nodeid_from_name(name)
+    if not nodeid:
+        return ip_list
+    nodeinfo = {}
+    nodeinfo = get_nodeinfo_from_cmaptool()
+    if not nodeinfo:
+        return ip_list
+    return nodeinfo[nodeid]
+
+
 def valid_nodeid(nodeid):
     from . import bootstrap
-    if not bootstrap.service_is_active('corosync.service'):
+    if not service_is_active('corosync.service'):
         return False
 
     for _id, _ in get_nodeinfo_from_cmaptool().items():
@@ -2250,11 +2160,6 @@ def valid_nodeid(nodeid):
     return False
 
 
-def is_unicast():
-    from . import corosync
-    return corosync.get_value("totem.transport") == "udpu"
-
-
 def get_nodeid_from_name(name):
     rc, out = get_stdout('crm_node -l')
     if rc != 0:
@@ -2264,4 +2169,466 @@ def get_nodeid_from_name(name):
         return res.group(1)
     else:
         return None
+
+
+def check_space_option_value(options):
+    if not isinstance(options, argparse.Namespace):
+        raise ValueError("Expected type of \"options\" is \"argparse.Namespace\", not \"{}\"".format(type(options)))
+
+    for opt in vars(options):
+        value = getattr(options, opt)
+        if isinstance(value, str) and len(value.strip()) == 0:
+            raise ValueError("Space value not allowed for dest \"{}\"".format(opt))
+
+
+def interface_choice():
+    _, out = get_stdout("ip a")
+    # should consider interface format like "ethx@xxx"
+    interface_list = re.findall(r'(?:[0-9]+:) (.*?)(?=: |@.*?: )', out)
+    return [nic for nic in interface_list if nic != "lo"]
+
+
+class IP(object):
+    """
+    Class to get some properties of IP address
+    """
+
+    def __init__(self, addr):
+        """
+        Init function
+        """
+        self.addr = addr
+
+    @property
+    def ip_address(self):
+        """
+        Create ipaddress instance
+        """
+        return ipaddress.ip_address(self.addr)
+
+    @property
+    def version(self):
+        """
+        Get IP address version
+        """
+        return self.ip_address.version
+
+    @classmethod
+    def is_mcast(cls, addr):
+        """
+        Check whether the address is multicast address
+        """
+        cls_inst = cls(addr)
+        return cls_inst.ip_address.is_multicast
+
+    @classmethod
+    def is_ipv6(cls, addr):
+        """
+        Check whether the address is IPV6 address
+        """
+        return cls(addr).version == 6
+
+    @classmethod
+    def is_valid_ip(cls, addr):
+        """
+        Check whether the address is valid IP address
+        """
+        cls_inst = cls(addr)
+        try:
+            cls_inst.ip_address
+        except ValueError:
+            return False
+        else:
+            return True
+
+    @property
+    def is_loopback(self):
+        """
+        Check whether the address is loopback address
+        """
+        return self.ip_address.is_loopback
+
+    @property
+    def is_link_local(self):
+        """
+        Check whether the address is link-local address
+        """
+        return self.ip_address.is_link_local
+
+
+class Interface(IP):
+    """
+    Class to get information from one interface
+    """
+
+    def __init__(self, ip_with_mask):
+        """
+        Init function
+        """
+        self.ip, self.mask = ip_with_mask.split('/')
+        super(__class__, self).__init__(self.ip)
+
+    @property
+    def ip_with_mask(self):
+        """
+        Get ip with netmask
+        """
+        return '{}/{}'.format(self.ip, self.mask)
+
+    @property
+    def ip_interface(self):
+        """
+        Create ip_interface instance
+        """
+        return ipaddress.ip_interface(self.ip_with_mask)
+
+    @property
+    def network(self):
+        """
+        Get network address
+        """
+        return str(self.ip_interface.network.network_address)
+
+    def ip_in_network(self, addr):
+        """
+        Check whether the addr in the network
+        """
+        return IP(addr).ip_address in self.ip_interface.network
+
+
+class InterfacesInfo(object):
+    """
+    Class to collect interfaces information on local node
+    """
+
+    def __init__(self, ipv6=False, second_heartbeat=False, custom_nic_list=[]):
+        """
+        Init function
+
+        On init process,
+        "ipv6" is provided by -I option
+        "second_heartbeat" is provided by -M option
+        "custom_nic_list" is provided by -i option
+        """
+        self.ip_version = 6 if ipv6 else 4
+        self.second_heartbeat = second_heartbeat
+        self._default_nic_list = custom_nic_list
+        self._nic_info_dict = {}
+
+    def get_interfaces_info(self):
+        """
+        Try to get interfaces info dictionary via "ip" command
+
+        IMPORTANT: This is the method that populates the data, should always be called after initialize
+        """
+        cmd = "ip -{} -o addr show".format(self.ip_version)
+        rc, out, err = get_stdout_stderr(cmd)
+        if rc != 0:
+            raise ValueError(err)
+
+        # format on each line will like:
+        # 2: enp1s0    inet 192.168.122.241/24 brd 192.168.122.255 scope global enp1s0\       valid_lft forever preferred_lft forever
+        for line in out.splitlines():
+            _, nic, _, ip_with_mask, *_ = line.split()
+            #TODO change this condition when corosync support link-local address
+            interface_inst = Interface(ip_with_mask)
+            if interface_inst.is_loopback or interface_inst.is_link_local:
+                continue
+            # one nic might configured multi IP addresses
+            if nic not in self._nic_info_dict:
+                self._nic_info_dict[nic] = []
+            self._nic_info_dict[nic].append(interface_inst)
+
+        if not self._nic_info_dict:
+            raise ValueError("No address configured")
+        if self.second_heartbeat and len(self._nic_info_dict) == 1:
+            raise ValueError("Cannot configure second heartbeat, since only one address is available")
+
+    @property
+    def nic_list(self):
+        """
+        Get interfaces name list
+        """
+        return list(self._nic_info_dict.keys())
+
+    @property
+    def interface_list(self):
+        """
+        Get instance list of class Interface
+        """
+        _interface_list = []
+        for interface in self._nic_info_dict.values():
+            _interface_list.extend(interface)
+        return _interface_list
+
+    @property
+    def ip_list(self):
+        """
+        Get IP address list
+        """
+        return [interface.ip for interface in self.interface_list]
+
+    @classmethod
+    def get_local_ip_list(cls, is_ipv6):
+        """
+        Get IP address list
+        """
+        cls_inst = cls(is_ipv6)
+        cls_inst.get_interfaces_info()
+        return cls_inst.ip_list
+
+    @classmethod
+    def ip_in_local(cls, addr):
+        """
+        Check whether given address was in one of local address
+        """
+        cls_inst = cls(IP.is_ipv6(addr))
+        cls_inst.get_interfaces_info()
+        return addr in cls_inst.ip_list
+
+    @property
+    def network_list(self):
+        """
+        Get network list
+        """
+        return list(set([interface.network for interface in self.interface_list]))
+
+    def _nic_first_ip(self, nic):
+        """
+        Get the first IP of specific nic
+        """
+        return self._nic_info_dict[nic][0].ip
+
+    def get_default_nic_list_from_route(self):
+        """
+        Get default nic list from route
+        """
+        if self._default_nic_list:
+            return self._default_nic_list
+
+        #TODO what if user only has ipv6 route?
+        cmd = "ip -o route show"
+        rc, out, err = get_stdout_stderr(cmd)
+        if rc != 0:
+            raise ValueError(err)
+        res = re.search(r'^default via .* dev (.*?) ', out)
+        if res:
+            self._default_nic_list = [res.group(1)]
+        else:
+            if not self.nic_list:
+                self.get_interfaces_info()
+            common_warn("No default route configured. Using the first found nic")
+            self._default_nic_list = [self.nic_list[0]]
+        return self._default_nic_list
+
+    def get_default_ip_list(self):
+        """
+        Get default IP list will be used by corosync
+        """
+        if not self._default_nic_list:
+            self.get_default_nic_list_from_route()
+        if not self.nic_list:
+            self.get_interfaces_info()
+
+        _ip_list = []
+        for nic in self._default_nic_list:
+            # in case given interface not exist
+            if nic not in self.nic_list:
+                raise ValueError("Failed to detect IP address for {}".format(nic))
+            _ip_list.append(self._nic_first_ip(nic))
+        # in case -M specified but given one interface via -i
+        if self.second_heartbeat and len(self._default_nic_list) == 1:
+            for nic in self.nic_list:
+                if nic not in self._default_nic_list:
+                    _ip_list.append(self._nic_first_ip(nic))
+                    break
+        return _ip_list
+
+    @classmethod
+    def ip_in_network(cls, addr):
+        """
+        Check whether given address was in one of local networks
+        """
+        cls_inst = cls(IP.is_ipv6(addr))
+        cls_inst.get_interfaces_info()
+        for interface_inst in cls_inst.interface_list:
+            if interface_inst.ip_in_network(addr):
+                return True
+        return False
+
+
+def check_file_content_included(source_file, target_file):
+    """
+    Check whether target_file includes contents of source_file
+    """
+    if not os.path.exists(source_file):
+        raise ValueError("File {} not exist".format(source_file))
+    if not os.path.exists(target_file):
+        return False
+
+    with open(target_file, 'r') as target_fd:
+        target_data = target_fd.read()
+    with open(source_file, 'r') as source_fd:
+        source_data = source_fd.read()
+    return source_data in target_data
+
+
+class ServiceManager(object):
+    """
+    Class to manage systemctl services
+    """
+    ACTION_MAP = {
+            "enable": "enable",
+            "disable": "disable",
+            "start": "start",
+            "stop": "stop",
+            "is_enabled": "is-enabled",
+            "is_active": "is-active",
+            "is_available": "list-unit-files"
+            }
+
+    def __init__(self, service_name, remote_addr=None):
+        """
+        Init function
+        """
+        self.service_name = service_name
+        self.remote_addr = remote_addr
+
+    def _do_action(self, action_type):
+        """
+        Actual do actions to manage service
+        """
+        if action_type not in self.ACTION_MAP.values():
+            raise ValueError("status_type should be {}".format('/'.join(list(self.ACTION_MAP.values()))))
+
+        cmd = "systemctl {} {}".format(action_type, self.service_name)
+        if self.remote_addr:
+            prompt_msg = "Run \"{}\" on {}".format(cmd, self.remote_addr)
+            rc, output, err = run_cmd_on_remote(cmd, self.remote_addr, prompt_msg)
+        else:
+            rc, output, err = get_stdout_stderr(cmd)
+        if rc != 0 and err:
+            raise ValueError("Run \"{}\" error: {}".format(cmd, err))
+        return rc == 0, output
+
+    @property
+    def is_available(self):
+        return self.service_name in self._do_action(self.ACTION_MAP["is_available"])[1]
+
+    @property
+    def is_enabled(self):
+        return self._do_action(self.ACTION_MAP["is_enabled"])[0]
+
+    @property
+    def is_active(self):
+        return self._do_action(self.ACTION_MAP["is_active"])[0]
+
+    def start(self):
+        self._do_action(self.ACTION_MAP["start"])
+
+    def stop(self):
+        self._do_action(self.ACTION_MAP["stop"])
+
+    def enable(self):
+        self._do_action(self.ACTION_MAP["enable"])
+
+    def disable(self):
+        self._do_action(self.ACTION_MAP["disable"])
+
+    @classmethod
+    def service_is_available(cls, name, remote_addr=None):
+        """
+        Check whether service is available
+        """
+        inst = cls(name, remote_addr)
+        return inst.is_available
+
+    @classmethod
+    def service_is_enabled(cls, name, remote_addr=None):
+        """
+        Check whether service is enabled
+        """
+        inst = cls(name, remote_addr)
+        return inst.is_enabled
+
+    @classmethod
+    def service_is_active(cls, name, remote_addr=None):
+        """
+        Check whether service is active
+        """
+        inst = cls(name, remote_addr)
+        return inst.is_active
+
+    @classmethod
+    def start_service(cls, name, enable=False, remote_addr=None):
+        """
+        Start service
+        """
+        inst = cls(name, remote_addr)
+        if enable:
+            inst.enable()
+        inst.start()
+
+    @classmethod
+    def stop_service(cls, name, disable=False, remote_addr=None):
+        """
+        Stop service
+        """
+        inst = cls(name, remote_addr)
+        if disable:
+            inst.disable()
+        inst.stop()
+
+    @classmethod
+    def enable_service(cls, name, remote_addr=None):
+        """
+        Enable service
+        """
+        inst = cls(name, remote_addr)
+        if inst.is_available and not inst.is_enabled:
+            inst.enable()
+
+    @classmethod
+    def disable_service(cls, name, remote_addr=None):
+        """
+        Disable service
+        """
+        inst = cls(name, remote_addr)
+        if inst.is_available and inst.is_enabled:
+            inst.disable()
+
+
+service_is_available = ServiceManager.service_is_available
+service_is_enabled = ServiceManager.service_is_enabled
+service_is_active = ServiceManager.service_is_active
+start_service = ServiceManager.start_service
+stop_service = ServiceManager.stop_service
+enable_service = ServiceManager.enable_service
+disable_service = ServiceManager.disable_service
+
+
+def package_is_installed(pkg, remote_addr=None):
+    """
+    Check if package is installed
+    """
+    cmd = "rpm -q --quiet {}".format(pkg)
+    if remote_addr:
+        # check on remote
+        prompt_msg = "Check whether {} is installed on {}".format(pkg, remote_addr)
+        rc, _, _ = run_cmd_on_remote(cmd, remote_addr, prompt_msg)
+    else:
+        # check on local
+        rc, _ = get_stdout(cmd)
+    return rc == 0
+
+
+@contextmanager
+def disable_exception_traceback():
+    """
+    All traceback information is suppressed and only the exception type and value are printed
+    """
+    default_value = getattr(sys, "tracebacklimit", 1000)  # `1000` is a Python's default value
+    sys.tracebacklimit = 0
+    yield
+    sys.tracebacklimit = default_value  # revert changes
 # vim:ts=4:sw=4:et:
