@@ -2,6 +2,7 @@
 # See COPYING for license information.
 
 import bz2
+import lzma
 import datetime
 import glob
 import gzip
@@ -26,6 +27,7 @@ import constants
 import crmsh.config
 from crmsh import msg as crmmsg
 from crmsh import utils as crmutils
+from crmsh import corosync
 
 
 class Tempfile(object):
@@ -76,8 +78,9 @@ def arch_logs(logf, from_time, to_time):
     """
     ret = []
     files = [logf]
-    #files += glob.glob(logf+"*[0-9z]")
-    for f in sorted(files, key=os.path.getctime):
+    files += glob.glob(logf+"*[0-9z]")
+    # like ls -t, newest first
+    for f in sorted(files, key=os.path.getmtime, reverse=True):
         # reset this var to check every file's format
         constants.GET_STAMP_FUNC = None
         res = is_our_log(f, from_time, to_time)
@@ -309,10 +312,14 @@ def collect_info():
 
     for p in process_list:
         p.join()
-    if not constants.SKIP_LVL:
-        sanitize()
 
-    for l in constants.EXTRA_LOGS.split():
+    logfile_list = []
+    corosync_log = corosync.get_value('logging.logfile')
+    if corosync_log:
+        logfile_list.append(corosync_log)
+    logfile_list += constants.EXTRA_LOGS.split()
+
+    for l in logfile_list:
         if not os.path.isfile(l):
             continue
         if l == constants.HA_LOG and l != constants.HALOG_F:
@@ -501,7 +508,7 @@ def dump_logset(logf, from_time, to_time, outf):
     else:
         out_string += print_logseg(oldest, from_time, 0)
         for f in mid_logfiles:
-            out_string += print_log(f)
+            out_string += print_logseg(f, 0, 0)
             log_debug("including complete %s logfile" % f)
         out_string += print_logseg(newest, 0, to_time)
 
@@ -591,19 +598,15 @@ def find_first_ts(data):
     return ts
 
 
-def filter_lines(logf, from_line, to_line=None):
+def filter_lines(data, from_line, to_line):
     out_string = ""
-    if not to_line:
-        to_line = sum(1 for l in open(logf, 'r', encoding='utf-8', errors='replace'))
-
     count = 1
-    with open(logf, 'r', encoding='utf-8', errors='replace') as f:
-        for line in f.readlines():
-            if count >= from_line and count <= to_line:
-                out_string += line
-            if count > to_line:
-                break
-            count += 1
+    for line in data.split('\n'):
+        if count >= from_line and count <= to_line:
+            out_string += line + '\n'
+        if count > to_line:
+            break
+        count += 1
     return out_string
 
 
@@ -717,43 +720,47 @@ def find_ssh_user():
         constants.SSH_USER = ssh_user
 
 
-def findln_by_time(logf, tm):
-    tmid = None
-    first = 1
-    last = sum(1 for l in open(logf, 'r', encoding='utf-8', errors='replace'))
+def findln_by_time(data, ts):
+    '''
+    Get line number of the specific time stamp
+    '''
+    data_list = data.split('\n')
+
+    first= 1
+    last= len(data_list)
+    time_middle = None
+
     while first <= last:
-        mid = (last+first)//2
+        middle = (last + first) // 2
         trycnt = 10
         while trycnt > 0:
-            res = line_time(logf, mid)
+            res = line_time(data_list, middle)
             if res:
-                tmid = int(res)
+                time_middle = res
                 break
-            log_debug("cannot extract time: %s:%d; will try the next one" % (logf, mid))
             trycnt -= 1
             # shift the whole first-last segment
-            prevmid = mid
-            while prevmid == mid:
+            prevmid = middle
+            while prevmid == middle:
                 first -= 1
                 if first < 1:
                     first = 1
                 last -= 1
                 if last < first:
                     last = first
-                prevmid = mid
-                mid = (last+first)//2
+                prevmid = middle
+                middle = (last + first) // 2
                 if first == last:
                     break
-        if not tmid:
-            log_warning("giving up on log...")
-            return
-        if int(tmid) > tm:
-            last = mid - 1
-        elif int(tmid) < tm:
-            first = mid + 1
+        if not time_middle:
+            return None
+        if time_middle > ts:
+            last = middle - 1
+        elif time_middle < ts:
+            first = middle + 1
         else:
             break
-    return mid
+    return middle
 
 
 def get_backtraces():
@@ -1178,10 +1185,12 @@ def is_our_log(logf, from_time, to_time):
     """
     check if the log contains a piece of our segment
     """
-    with open(logf, 'r', encoding='utf-8', errors="replace") as fd:
-        data = fd.read()
-        first_time = find_first_ts(head(10, data))
-        last_time = find_first_ts(tail(10, data))
+    data = read_from_file(logf)
+    if not data:
+        log_debug("Found empty file \"{}\"; exclude".format(logf))
+        return 0
+    first_time = find_first_ts(head(10, data))
+    last_time = find_first_ts(tail(10, data))
 
     if (not first_time) or (not last_time):
         if os.stat(logf).st_size > 0:
@@ -1198,13 +1207,12 @@ def is_our_log(logf, from_time, to_time):
         return 0  # don't include this log
 
 
-def line_time(logf, line_num):
-    ts = None
-    with open(logf, 'r', encoding='utf-8', errors='replace') as fd:
-        line_res = head(line_num, fd.read())
-        if line_res:
-            ts = get_ts(line_res[-1])
-    return ts
+
+def line_time(data_list, line_num):
+    '''
+    Get time stamp of the specific line
+    '''
+    return get_ts(data_list[line_num-1])
 
 
 def load_ocf_dirs():
@@ -1363,38 +1371,24 @@ def print_log(logf):
 
 
 def print_logseg(logf, from_time, to_time):
-    cat = find_decompressor(logf)
-    if cat != "cat":
-        tmp = create_tempfile()
-        add_tempfiles(tmp)
-
-        cmd = "%s %s > %s" % (cat, logf, tmp)
-        code, out, err = crmutils.get_stdout_stderr(cmd)
-        if code != 0:
-            log_fatal("maybe disk full: %s" % err)
-        sourcef = tmp
-    else:
-        sourcef = logf
-        tmp = ""
+    data = read_from_file(logf)
 
     if from_time == 0:
-        FROM_LINE = 1
+        from_line = 1
     else:
-        FROM_LINE = findln_by_time(sourcef, from_time)
-
-    if not FROM_LINE:
-        log_warning("couldn't find line for time %d; corrupt log file?" % from_time)
-        return ""
-
-    TO_LINE = ""
-    if to_time != 0:
-        TO_LINE = findln_by_time(sourcef, to_time)
-        if not TO_LINE:
-            log_warning("couldn't find line for time %d; corrupt log file?" % to_time)
+        from_line = findln_by_time(data, from_time)
+        if from_line is None:
             return ""
 
-    log_debug("including segment [%s-%s] from %s" % (FROM_LINE, TO_LINE, sourcef))
-    return dump_log(sourcef, FROM_LINE, TO_LINE)
+    if to_time == 0:
+        to_line = len(data.split('\n'))
+    else:
+        to_line = findln_by_time(data, to_time)
+        if to_line is None:
+            return ""
+
+    log_debug("Including segment [{}-{}] from {}".format(from_line, to_line, logf))
+    return filter_lines(data, from_line, to_line)
 
 
 def ra_build_info():
@@ -1418,48 +1412,48 @@ def sanitize():
     """
     replace sensitive info with '****'
     """
-    workdir = constants.WORKDIR
-    conf = os.path.join(workdir, constants.B_CONF)
-    if os.path.isfile(conf):
-        sanitize_one(conf)
-    cib_f = os.path.join(workdir, constants.CIB_F)
-    rc = 0
-    for f in [cib_f] + glob.glob(os.path.join(workdir, "pengine", "*")):
-        if os.path.isfile(f):
-            if constants.DO_SANITIZE == 1:
-                sanitize_one(f)
-            else:
-                rc = sanitize_one(f, "test")
-    if rc != 0:
-        log_warning("some PE or CIB files contain possibly sensitive data")
-        log_warning("you may not want to send this report to a public mailing list")
+    log_debug("Check or replace sensitive info from cib, pe and log files")
+
+    get_sensitive_key_value_list()
+
+    work_dir = constants.WORKDIR
+    file_list = []
+    for (dirpath, dirnames, filenames) in os.walk(work_dir):
+        for _file in filenames:
+            file_list.append(os.path.join(dirpath, _file))
+
+    for f in [item for item in file_list if os.path.isfile(item)]:
+        rc = sanitize_one(f)
+        if rc == 1:
+            log_warning("Some PE/CIB/log files contain possibly sensitive data")
+            log_warning("Using \"-s\" option can replace sensitive data")
+            break
 
 
-def sanitize_one(in_file, mode=None):
-    open_ = None
-    if re.search("gz$", in_file):
-        open_ = gzip.open
-    elif re.search("bz2$", in_file):
-        open_ = bz2.BZ2File
-    else:
-        open_ = open
-    with open_(in_file, 'r') as f:
-        data = f.read()
+def sanitize_one(in_file):
+    """
+    Open the file, replace sensitive string and write back
+    """
+    data = read_from_file(in_file)
+    if not data:
+        return
+    if not include_sensitive_data(data):
+        return
+    if not constants.DO_SANITIZE:
+        return 1
+    log_debug("Replace sensitive info for {}".format(in_file))
+    write_to_file(in_file, sub_sensitive_string(data))
 
-    if mode == "test":
-        if sub_string_test(data):
-            return 1
+
+def parse_sanitize_rule(rule_string):
+    for rule in rule_string.split():
+        if ':' in rule:
+            key, value = rule.split(':')
+            if value != "raw":
+                log_fatal("For sanitize_pattern {}, option should be \"raw\"".format(key))
+            constants.SANITIZE_RULE_DICT[key] = value
         else:
-            return 0
-
-    ref = create_tempfile()
-    add_tempfiles(ref)
-    touch_r(in_file, ref)
-
-    with open_(in_file, 'w') as f:
-        f.write(sub_string(data))
-
-    touch_r(ref, in_file)
+            constants.SANITIZE_RULE_DICT[rule] = None
 
 
 def say_ssh_user():
@@ -1548,42 +1542,23 @@ def start_slave_collector(node, arg_str):
                     log_warning(err)
                 break
 
+    compress_data = ""
+    for data in out.split('\n'):
+        if data.startswith(constants.COMPRESS_DATA_FLAG):
+            # hb_report data from collector
+            compress_data = data.lstrip(constants.COMPRESS_DATA_FLAG)
+        else:
+            # log data from collector
+            print(data)
+
     cmd = r"(cd {} && tar xf -)".format(constants.WORKDIR)
-    # typeof out here will always "str"
-    # input_s of get_stdout will always need bytes
-    # different situation depend on whether found pe file
-    if out.startswith("b'"):
-        crmutils.get_stdout(cmd, input_s=eval(out))
-    else:
-        crmutils.get_stdout(cmd, input_s=out.encode('utf-8'))
+    crmutils.get_stdout(cmd, input_s=eval(compress_data))
 
 
 def str_to_bool(v):
     return v.lower() in ["true"]
 
 
-def sub_string(in_string,
-               pattern=constants.SANITIZE,
-               sub_pattern=' value=".*" ',
-               repl=' value="******" '):
-    res_string = ""
-    pattern_string = re.sub(" ", "|", pattern)
-    for line in in_string.split('\n')[:-1]:
-        if re.search('name="%s"' % pattern_string, line):
-            res_string += re.sub(sub_pattern, repl, line) + '\n'
-        else:
-            res_string += line + '\n'
-    return res_string
-
-
-def sub_string_test(in_string, pattern=constants.SANITIZE):
-    pattern_string = re.sub(" ", "|", pattern)
-    for line in crmutils.to_ascii(in_string).split('\n'):
-        if re.search('name="%s"' % pattern_string, line):
-            return True
-    return False
-
-
 def sys_info():
     """
     some basic system info and stats
@@ -1649,11 +1624,29 @@ def time_status():
     time_f = os.path.join(constants.WORKDIR, constants.TIME_F)
     crmutils.str2file(out_string, time_f)
 
+
+def dump_D_process():
+    '''
+    dump D-state process stack
+    '''
+    out_string = ""
+    _, out, _ = crmutils.get_stdout_stderr("ps aux|awk '$8 ~ /^D/{print $2}'")
+    len_D_process = len(out.split('\n')) if out else 0
+    out_string += "Dump D-state process stack: {}\n".format(len_D_process)
+    if len_D_process == 0:
+        return out_string
+    for pid in out.split('\n'):
+        _, cmd_out, _ = crmutils.get_stdout_stderr("cat /proc/{}/comm".format(pid))
+        out_string += "pid: {}     comm: {}\n".format(pid, cmd_out)
+        _, stack_out, _ = crmutils.get_stdout_stderr("cat /proc/{}/stack".format(pid))
+        out_string += stack_out + "\n\n"
+    return out_string
+
+
 def dump_ocfs2():
     ocfs2_f = os.path.join(constants.WORKDIR, constants.OCFS2_F)
     with open(ocfs2_f, "w") as f:
-        #dump all tasks stack into dmesg
-        os.system("echo t > /proc/sysrq-trigger")
+        f.write(dump_D_process())
 
         cmds = [ "dmesg",  "ps -efL", "lsof",
                 "lsblk -o 'NAME,KNAME,MAJ:MIN,FSTYPE,LABEL,RO,RM,MODEL,SIZE,OWNER,GROUP,MODE,ALIGNMENT,MIN-IO,OPT-IO,PHY-SEC,LOG-SEC,ROTA,SCHED,MOUNTPOINT'",
@@ -1670,6 +1663,7 @@ def dump_ocfs2():
             f.write("# %s\n"%(cmd))
             f.write(out)
 
+
 def touch_dc():
     if constants.SKIP_LVL:
         return
@@ -1746,4 +1740,89 @@ def which(prog):
     else:
         return False
 
+
+def get_open_method(infile):
+    file_type_open_dict = {
+            "gz": gzip.open,
+            "bz2": bz2.open,
+            "xz": lzma.open
+            }
+    try:
+        _open = file_type_open_dict[infile.split('.')[-1]]
+    except KeyError:
+        _open = open
+    return _open
+
+
+def read_from_file(infile):
+    data = None
+    _open = get_open_method(infile)
+    with _open(infile, 'rt', encoding='utf-8', errors='replace') as f:
+        data = f.read()
+    return crmutils.to_ascii(data)
+
+
+def write_to_file(tofile, data):
+    _open = get_open_method(tofile)
+    with _open(tofile, 'w') as f:
+        if _open == open:
+            f.write(data)
+        else:
+            f.write(data.encode('utf-8'))
+
+
+def get_sensitive_key_value_list():
+    """
+    For each defined sanitize rule, get the sensitive value or key list
+    """
+    for key, value in constants.SANITIZE_RULE_DICT.items():
+        try:
+            if value == "raw":
+                constants.SANITIZE_VALUE_RAW += extract_sensitive_value_list(key)
+            else:
+                constants.SANITIZE_VALUE_CIB += extract_sensitive_value_list(key)
+                constants.SANITIZE_KEY_CIB.append(key.strip('.*?')+'.*?')
+        except (FileNotFoundError, EOFError) as e:
+            log_warning(e)
+
+
+def extract_sensitive_value_list(rule):
+    """
+    Extract sensitive value from cib.xml
+    """
+    cib_file = os.path.join(constants.WORKDIR, constants.WE, constants.CIB_F)
+    if not os.path.exists(cib_file):
+        raise FileNotFoundError("File {} was not collected".format(constants.CIB_F))
+
+    with open(cib_file) as fd:
+        data = fd.read()
+    if not data:
+        raise EOFError("File {} is empty".format(cib_file))
+
+    value_list = re.findall(r'name="({})" value="(.*?)"'.format(rule.strip('?')+'?'), data)
+    return [value[1] for value in value_list]
+
+
+def include_sensitive_data(data):
+    """
+    Check whether contain sensitive data
+    """
+    if constants.SANITIZE_VALUE_RAW or constants.SANITIZE_VALUE_CIB:
+        return True
+    return False
+
+
+def sub_sensitive_string(data):
+    """
+    Do the replace job
+
+    For the raw sanitize_pattern option, replace exactly the value
+    For the key:value nvpair sanitize_pattern, replace the value in which line contain the key
+    """
+    result = data
+    if constants.SANITIZE_VALUE_RAW:
+        result = re.sub(r'\b({})\b'.format('|'.join(constants.SANITIZE_VALUE_RAW)), "******", data)
+    if constants.SANITIZE_VALUE_CIB:
+        result = re.sub('({})({})'.format('|'.join(constants.SANITIZE_KEY_CIB), '|'.join(constants.SANITIZE_VALUE_CIB)), '\\1******', result)
+    return result
 # vim:ts=4:sw=4:et:
