@@ -1,4 +1,6 @@
-#!/usr/bin/env python
+#!/usr/bin/python3
+from __future__ import unicode_literals
+from builtins import str
 import os
 import pwd
 import hashlib
@@ -11,21 +13,25 @@ PACKAGES = ['booth', 'cluster-glue', 'corosync', 'crmsh', 'csync2', 'drbd',
             'haproxy', 'hawk', 'libdlm', 'libqb', 'ocfs2', 'ocfs2-tools',
             'pacemaker', 'pacemaker-mgmt', 'resource-agents', 'sbd']
 
+
 def rpm_info():
     return crm_script.rpmcheck(PACKAGES)
 
+
 def logrotate_info():
     return {}
 
+
 def get_user():
     return pwd.getpwuid(os.getuid()).pw_name
 
+
 def sys_info():
     sysname, nodename, release, version, machine = os.uname()
-    #The first three columns measure CPU and IO utilization of the
-    #last one, five, and 15 minute periods. The fourth column shows
-    #the number of currently running processes and the total number of
-    #processes. The last column displays the last process ID used.
+    # The first three columns measure CPU and IO utilization of the
+    # last one, five, and 15 minute periods. The fourth column shows
+    # the number of currently running processes and the total number of
+    # processes. The last column displays the last process ID used.
     system, node, release, version, machine, processor = platform.uname()
     distname, distver, distid = platform.linux_distribution()
     hostname = os.uname()[1]
@@ -49,6 +55,7 @@ def sys_info():
             'loadavg': loadavg[2]  # 15 minute average
             }
 
+
 def disk_info():
     rc, out, err = crm_script.call(['df'], shell=False)
     if rc == 0:
@@ -62,6 +69,7 @@ def disk_info():
         return disk_use
     return []
 
+
 # configurations out of sync
 
 FILES = [
@@ -79,8 +87,8 @@ def files_info():
     for f in FILES:
         if os.path.isfile(f):
             try:
-                ret[f] = hashlib.sha1(open(f).read()).hexdigest()
-            except IOError, e:
+                ret[f] = hashlib.sha1(open(f).read().encode('utf-8')).hexdigest()
+            except IOError as e:
                 ret[f] = "error: %s" % (e)
         else:
             ret[f] = ""
@@ -96,5 +104,5 @@ try:
         'files': files_info()
     }
     crm_script.exit_ok(data)
-except Exception, e:
+except Exception as e:
     crm_script.exit_fail(str(e))
