@@ -1,10 +1,4 @@
 #!/bin/sh
-
-unit_tests() {
-	echo "** Unit tests"
-	./test/run
-}
-
 configure() {
 	echo "** Autogen / Configure"
 	./autogen.sh
@@ -14,6 +8,7 @@ configure() {
 make_install() {
 	echo "** Make / Install"
 	make install
+	make install-crmconfDATA prefix=
 }
 
 regression_tests() {
@@ -32,13 +27,11 @@ case "$1" in
 		configure
 		make_install
 		exit $?;;
-	bootstrap|qdevice|hb_report|resource)
+	bootstrap|qdevice|hb_report|resource|geo|configure|constraints)
 		functional_tests $1 $2
 		exit $?;;
-	*)
-		unit_tests
-		rc_unittest=$?
+	*|original)
 		configure
 		make_install
-		regression_tests && exit $rc_unittest;;
+		regression_tests;;
 esac
